@@ -1,14 +1,25 @@
 package Config::Simple;
-# $Id: Simple.pm,v 3.0 2002/11/03 02:59:22 sherzodr Exp $
+
+# $Id: Simple.pm,v 3.10 2002/12/17 16:35:08 sherzodr Exp $
 
 use strict;
 use Carp 'croak';
 use Fcntl (':DEFAULT', ':flock');
+use Text::ParseWords;
 
-use vars qw($VERSION);
+use vars qw($VERSION $DEFAULTNS $USEQQ);
 
-($VERSION) = '$Revision: 3.0 $' =~ m/Revision:\s*(\S+)/;
+sub import {
+    (undef) = shift;
+    for ( @_ ) {
+        $USEQQ = ($_ eq '-strict') || next;
+    }
+}
 
+($VERSION) = '$Revision: 3.10 $' =~ m/Revision:\s*(\S+)/;
+
+# Default namespace as suggested by Ruslan U. Zakirov <cubic@wr.miee.ru>
+$DEFAULTNS = "default";
 
 sub new {
     my $class = shift;
@@ -22,10 +33,11 @@ sub new {
             filename => 0,
         },
         _data    => { },
+        _status  => 0,
 
     };
 
-    # If there's only one argument, consider it's the filename
+    # If there's only one argument, consider it the filename
     if ( @_ == 1 ) {
         $self->{_options}->{filename} = $_[0];
 
@@ -39,10 +51,13 @@ sub new {
             @_, };
     }
 
+    if ( $self->{_options}->{autosave} and ! $self->{_options}->{filename} ) {
+        croak "Cannot create autosave without a file name";
+    }
     bless($self, $class);
 
     if ( $self->{_options}->{filename} ) {
-        $self->read();
+        $self->read($self->{_options}->{filename});
     }
     return $self;
 }
@@ -52,7 +67,7 @@ sub new {
 sub DESTROY {
     my $self = shift;
 
-    if ( $self->{_options}->{autosave} ) {
+    if ( $self->{_options}->{autosave} && $self->{_options}->{filename}) {
         $self->write();
     }
 
@@ -76,31 +91,58 @@ sub read {
     }
 
     my $filename = $self->{_options}->{filename};
-
-    unless ( sysopen(CFG, $filename, O_RDWR|O_CREAT) ) {
-        croak "Couldn't read $filename: $!";
+    unless ( defined $filename ) {
+        croak "No file name specified";
     }
-    $self->{_fh} = \*CFG;
+
+    unless ( sysopen(CFG, $filename, O_RDONLY) ) {
+        croak "Couldn't read $filename: $!";
+    }    
     unless ( flock(CFG, LOCK_SH) ) {
         croak "Couldn't LOCK_SH $filename: $!";
     }
 
     # whitespace support (\s*) added by Michael Caldwell <mjc@mjcnet.com>
     # date: Tuesday, May 14, 2002
-    my ($blockname, $afterdot);
+
+    # default namespace suggestion and partial patch submitted by
+    # Ruslan U. Zakirov <cubic@wr.miee.ru>
+    # date: Sat, Nov 09, 2002
+
+    my $afterdot= 0;
+    my $ns      = $DEFAULTNS;
+    my $data    = {}; # to help the while() loop look less hairy
+
     while ( <CFG> ) {
-        $afterdot                   and $self->{_after_dot} .= $_, next;
-        /^(\n|\#|;)/                and next;
-        /\s*\[([^\]]+)\]\s*/        and $blockname = lc($1), next;
-        /\s*([^=]+?)\s*=\s*(.+)/    and $self->{_data}->{$blockname}->{lc($1)}=$self->_decode($2), next;
-        /^\./                       and $afterdot=1, next;
+        # If we're processing the lines after the single dot (.),
+        # store everything as it is
+        $afterdot                and $self->{_after_dot} .= $_, next;
+
+        # If we come accross empty lines or comments, skip them
+        /^(\n|\#|;)/             and next;
+
+        # If we came this far, we're guaranteed that the current line
+        # we're working on is either a block declaration like [something]
+        # or key=value pairs, or may be even a single dot (.) character.
+        
+        /\s*\[([^\]]+)\]\s*/     and $ns = lc($1), next;        
+        /\s*([^=]+?)\s*=\s*(.*)/ and $USEQQ ? 
+                                    ($data->{$ns}->{lc($1)} = (quotewords('\s+',0,$self->_decode($2)))[0]) :
+                                                ($data->{$ns}->{lc($1)} = $self->_decode($2)), next;
+                                        
+                                    
+
+        /^\./                    and $afterdot=1, next;
 
         # If we came this far, something smells fishy around here
         croak "Syntax error on line $.:'$_'";
     }
 
-    unless ( flock(CFG, LOCK_UN) ) {
-        croak "Couldn't LOCK_UN $filename: $!";
+    # don't forget to assign the $data to object
+    $self->{_data} = $data;
+
+    unless ( close(CFG) ) {
+        croak "Couldn't close $filename: $!";
     }
 }
 
@@ -131,7 +173,7 @@ sub _get_param {
     }
 
     unless ( defined $param ) {
-        croak "_get_param(): RTFM!";
+        croak "_get_param(): Invalid parameter format";
     }
 
     return $self->{_data}->{$blockname}->{$param};
@@ -149,7 +191,7 @@ sub _set_param {
     my ($self, $blockname, $param, $value) = @_;
 
     if ( @_ < 3 ) {
-        croak "_set_param(): RTFM!";
+        croak "_set_param(): Insufficient arguments";
     }
 
     # if the last argument, value is missing, treat $param as $value,
@@ -160,7 +202,7 @@ sub _set_param {
     }
 
     unless ( $blockname ) {
-        croak "_set_param(): RTFM!";
+        croak "_set_param(): Invalid parameter format";
     }
 
     $self->{_data}->{lc($blockname)}->{lc($param)} = $value;
@@ -262,7 +304,48 @@ sub param {
         return $self->_set_param(@_);
     }
 
-    croak "Config::Simple->param() usage was incorrect!";
+    croak "param(): usage incorrect";
+}
+
+
+
+
+
+
+sub block {
+    my $self = shift;
+
+    if ( @_ ) {
+        return $self->_get_block(@_);
+    }
+
+    return keys %{$self->{_data}};
+}
+
+
+
+
+
+
+
+
+sub delete {
+    my $self = shift;
+
+    unless (@_) {
+        croak "delete(): Insufficient parameters";
+    }
+
+    my ($block, $param) = split (/\./, $_[0]);
+
+    # if block and $params are defined, we should clear
+    # only that particular key. Otherwise, we take it as a
+    # name of a whole block to be deleted
+    if ( defined($block) && defined($param) ) {
+        return delete $self->{_data}->{$block}->{$param};
+    }
+
+    return delete $self->{_data}->{ $_[0] };
 }
 
 
@@ -270,55 +353,66 @@ sub param {
 sub write {
     my ($self, $new_file) = @_;
 
-    my $data    = $self->{_data}    or return;
-    my $fh      = $self->{_fh}      or return;
-    my $file    = $self->{_options}->{filename};
+    # generating the content of the configuration file as a string    
+    my $contents = $self->write_string() or return;    
+    my $file    = $new_file || $self->{_options}->{filename};
 
-    if ( defined $new_file ) {
-        unless ( sysopen (NEWFILE, $new_file, O_WRONLY|O_CREAT|O_TRUNC, 0666) ) {
-            croak "Couldn't open $new_file: $!";
-        }
-        $fh = \*NEWFILE;
+    unless ( defined $file ) {
+        croak "Don't which where to write into";
+    }    
+    
+    unless ( sysopen (FILE, $file, O_WRONLY|O_CREAT|O_TRUNC, 0666) ) {
+        croak "Couldn't open $file: $!";
     }
-
-    unless ( flock ($fh, LOCK_EX) ) {
+    
+    unless ( flock (FILE, LOCK_EX) ) {
         croak "Couldn't LOCK_EX $file: $!";
-    }
+    }    
+    print FILE $contents;    
 
-    unless ( truncate($fh, 0) ) {
-        croak "Couldn't truncate $file: $!";
+    unless ( flock(FILE, LOCK_UN) ) {
+        croak "Couldn't unlock $file: $!";
     }
+    
+    unless ( close(FILE) ) {
+        croak "Couldn't close $new_file: $!";
+    }    
+}
 
-    unless ( seek($fh, 0, 0) ) {
-        croak "Couldn't seek to the start of the file: $!";
-    }
 
-    print $fh "; Maintained by Config::Simple/$VERSION\n";
-    print $fh '; ', "-" x 35, "\n\n\n";
+
+
+
+sub write_string {
+    my $self = shift;
+
+    my $data = $self->{_data} or return;
+
+    # Array to hold file lines
+    my @contents = ();
+
+    push @contents, "# Maintained by Config::Simple/$VERSION";
+    push @contents, '# ' . ("-" x 35), '', '';
 
     while ( my ($blockname, $block) = each %{$data} ) {
-        print $fh "[$blockname]\n";
+        push @contents, "[$blockname]";
         while ( my ($key, $value) = each %{$block} ) {
             $value = $self->_encode($value);
-            print $fh "$key=$value\n";
+            push @contents, "$key=$value";
         }
-        print $fh "\n\n";
+        push @contents, '', '';
     }
 
     if ( $self->{_after_dot} ) {
-        print $fh ".\n", $self->{_after_dot};
+        push @contents, ".", $self->{_after_dot};
     }
 
-    unless ( flock($fh, LOCK_UN) ) {
-        croak "Couldn't unlock $file: $!";
-    }
+    # Merge and return the file contents
+    return join "\n", @contents;
+ }
+
+
 
-    if ( defined $new_file ) {
-        unless ( close($fh) ) {
-            croak "Couldn't close $new_file: $!";
-        }
-    }
-}
 
 
 sub _encode {
@@ -414,8 +508,6 @@ Config::Simple - Simple Configuration File class
     database=test
 
 
-
-
     # In your program
 
     use Config::Simple;
@@ -444,7 +536,65 @@ help you with it.
 
 =head1 REVISION
 
-This manual refers to $Revision: 3.0 $
+This manual refers to $Revision: 3.10 $
+
+=head1 CONFIGURATION FILE SYNTAX
+
+Syntax of the configuration file is similar to windows .ini files, where
+configuration variables and their values are seperated with '=' sign, 
+each set belongind to a specific namespace (block):
+
+	[block]
+	var1=value1
+	var2=value2
+
+If the block is missing, or any of the key=value pairs are encountered
+without prior block declaration, they will be assigned to a virtual block.
+Name of the virtual block is controlled with B<$Config::Simple::DEFAULTNS>
+variable:
+
+    use Config::Simple;
+    $Config::Simple::DEFAULTNS = "root";
+    $cfg = new Config::Simple("some.cfg");
+
+If you do not explicitly assign a namespace, "default" is implied. 
+
+By default, Config::Simple treats everything after the '=' to the end of the
+line as configuration value (spaces trimmed). Unfortunately, other configuration
+file parsing utilities do not quite aggree with the idea, and assert if the
+value contains non-alphanumerics, they should be enclosed in double quotes (").
+Config::Simple supports this syntax as well. If you intend to use this syntax,
+you should enable "-strict" switch like so:
+
+    use Config::Simple qw/-strict/;
+
+Now Config::Simple expects to see any values that contain non-alphanumeric
+characters in double quotes. Double quotes to be used inside the value should
+be escaped with \ (backslash):
+
+    [default]
+    nick = sherzodR
+    name = "Sherzod Ruzmetov"
+    url  = "http://author.ultracgis.com"
+    email= "sherzodr@cpan.org"
+    quoted = "My favorite quote is \"Learn as if you were to live forever\""
+    age  = 22
+
+Enabling "-strict" switch is also useful to keep trailing and leading spaces in 
+configuration values:
+
+    [default]
+    signature = "            Foo Bar                   "
+
+Lines starting with '#' or ';' to the end of the line are considered comments,
+thus ignored while parsing. Line, containing a single dot is the logical end
+of the configuration file (doesn't necessaryily have to be the physical end though ).
+So everything after that line is also ignored. 
+
+Note, when you ask Config::Simple to save the changes back, all the comments
+will be discarded, but everything after that final dot is stored back as it was. 
+
+I admit, keeping the comments would be quite useful too. May be in subsequent releases.
 
 =head1 CONSTRUCTOR
 
@@ -578,9 +728,18 @@ We can access the above block like so:
 
 Used to create a new block or redefine the existing one.
 
-
 =back
 
+
+=item block()
+
+Returns the list of all the avialable blocks in the configuration file.
+If used with an argument, returns the content of that particular block
+in the form of hashref:
+
+    my @blocks = $cfg->block();
+
+
 =item *
 
 write() - saves the modifications to the configuration file. Config::Simple
@@ -595,6 +754,13 @@ before making any changes to it:
     $cfg->write('some.cfg.bak');        # creating backup copy
                                         # before updating the contents
 
+
+=item *
+
+write_string() - creates and returns the content of the configuration file as a string,
+instead of writing it into a file. The string returned by write_string() is guaranteed
+to be the same as what is written into a file.
+
 =item *
 
 encoder() - sets a new encoder to be used in the form of coderef. This encoder
@@ -611,7 +777,6 @@ Alternatively, you can define the decoder as an argument to constructor ( see L<
 
 autosave() - sets autosave value (see L<new()>)
 
-
 =item *
 
 dump() - dumps the object data structure either to STDOUT or into a filename which
@@ -634,6 +799,20 @@ Added witespace support in the configuration files, which enables custom identat
 
 Fixed the bugs in the TIEHASH method.
 
+=item Ruslan U. Zakirov <cubic@wr.miee.ru>
+
+Default namespace suggestion and patch.
+
+=item Adam Kennedy <cpan@ali.as>  
+
+Added a write_string() method, for getting the file as a string.
+Fix for the case when the value is ''
+
+=item Gavin Brown <gavin.brown@centralnic.com>
+
+Proposed quoted values feature. This made configuration files created by
+Config::Simple compatible with those expected by PHP's parse_ini_file() function.
+
 =back
 
 =head1 AUTHOR
